/**
  * ircommander.c
  * 
  * Dit programma leest de IR-signalen van panasonic-afstandsbedieningen
	* en zet de bijbehorende tekst op de standaarduitvoer zoals opgegeven in
	* config/ircommander.conf
	*
	* ircommander.conf bevat ook de instelling voor het live toevoegen
	* van commando's en de tolerantie voor het lang indrukken van een knop
  * 
	* Het programma beschouwt het PRU-geheugen als een uint64-vector
	* van 256 elementen voorafgegaan door een uint32 voor de lengte.
  */

#include <stdio.h>
#include <sys/mman.h>
#include <unistd.h>
#include <malloc.h>
#include <string.h>
#include <stdbool.h>
#include <prussdrv.h>
#include <pruss_intc_mapping.h>
#include <glib.h>

#ifdef DEBUG
#define DEBUG_PRINTF(FORMAT, ...) fprintf(stderr, FORMAT, ## __VA_ARGS__)
#else
#define DEBUG_PRINTF(FORMAT, ...)
#endif

// Flexibele arrays (C99) zijn nodig om de buffer(-code) simpel te houden
typedef struct _SimpleBuffer {
	unsigned int write_pos;
	unsigned int read_pos;
	unsigned long long array[];
} SimpleBuffer;

void poll(GKeyFile *keyfile, int tolerance, bool edit_mode, SimpleBuffer *buffer);
bool assign_command(GKeyFile *keyfile, char *keycode_string);
bool check_repetition(const unsigned long long keycode, const int tolerance);

int main() {
	tpruss_intc_initdata pruss_intc_initdata = PRUSS_INTC_INITDATA;
	GKeyFile *keyfile = g_key_file_new();
	GKeyFileFlags flags = G_KEY_FILE_NONE;
	GError *error;
	bool edit_mode;
	int repetition_tolerance;
	SimpleBuffer *buffer;

	// Laad het configuratiebestand in
	if (!g_key_file_load_from_file(keyfile, "../config/ircommander.conf", flags, &error))
		fprintf(stderr, error->message);
	
	edit_mode = g_key_file_get_boolean(keyfile, "General", "assignNewCommands", NULL);
	repetition_tolerance = g_key_file_get_integer(keyfile, "General", "repetitionTolerance", NULL);

	prussdrv_init();

	// Open de PRU zonder speciale interrupts en laad de mmap
	if (prussdrv_open(PRU_EVTOUT_0))
	{
		fprintf(stderr, "prussdrv_open open failed\n");
		return 1;
	}

	// Start de interrupt controller
	prussdrv_pruintc_init(&pruss_intc_initdata);
	
	// Maak een koppeling met het geheugen van de PRU
	prussdrv_map_prumem(PRUSS0_PRU0_DATARAM, (void**)(&buffer));
	
	// Zet de indices op 0
	buffer->write_pos = 0;
	buffer->read_pos = 0;
	
	// Zet het PRU-programma aan
	prussdrv_exec_program(PRU0, "./ircommander.bin");
	
	// Start het pollen van het PRU-geheugen
	poll(keyfile, repetition_tolerance, edit_mode, buffer);

	// Zet de PRU uit en stop de driver
	prussdrv_pru_disable(PRU0); 
	prussdrv_exit();
	
	return 0;
}

void poll(GKeyFile *keyfile, int tolerance, bool edit_mode, SimpleBuffer *buffer) {
	unsigned long long keycode;
	char keycode_string[20];
	char *command;
	
	while (1) {
		// Wacht op een interrupt van de PRU
    prussdrv_pru_wait_event(PRU_EVTOUT_0);
    prussdrv_pru_clear_event(PRU_EVTOUT_0, PRU0_ARM_INTERRUPT);
		
		// Lees alle nieuwe commando's
		while(buffer->read_pos != buffer->write_pos) {
			buffer->read_pos = (buffer->read_pos+1) & ((1<<8)-1);
			keycode = buffer->array[buffer->read_pos];
			sprintf(keycode_string, "%llX", keycode);

			// Vergeef de gebruiker die de knop iets te lang indrukt
			if (!check_repetition(keycode, tolerance))
				continue;
			
			DEBUG_PRINTF("Nieuwe keycode %s.\n", keycode_string);

			if (!g_key_file_has_key(keyfile, "Commands", keycode_string, NULL) && edit_mode)
				assign_command(keyfile, keycode_string);
			
			if (g_key_file_has_key(keyfile, "Commands", keycode_string, NULL)) {
				command = g_key_file_get_string(keyfile, "Commands", keycode_string, NULL);
			
				// Voer het commando uit en forceer het flushen van de uitvoer
				printf("%s\n", command);
				fflush(stdout);
				
				DEBUG_PRINTF("Commando \"%s\" verstuurd.\n", command);
				g_free(command);
			}
		}
		
		DEBUG_PRINTF("read_pos: %d, write_pos: %d, adressen: %X, %X, %X\n", buffer->read_pos, buffer->write_pos, &(buffer->read_pos), &(buffer->write_pos), buffer->array);
	}
}

bool assign_command(GKeyFile *keyfile, char *keycode_string) {
	char *new_command = NULL;
	char *newline_address;
	size_t command_length;
	
	fprintf(stderr, "Voer het commando voor %s in:\n", keycode_string);
	
	if (getline(&new_command, &command_length, stdin) != -1) {
		// Haal de newline uit het commando
		if (newline_address = strchr(new_command, '\n'))
			*newline_address = '\0';

		// Voeg het commando to aan de database en sla deze direct op
		DEBUG_PRINTF("\"[Commands] %s = %s\" ingevoerd.\n", keycode_string, new_command);
		g_key_file_set_string(keyfile, "Commands", keycode_string, new_command);
		g_key_file_save_to_file(keyfile, "../config/ircommander.conf", NULL);
		
		free(new_command);
	}
}

bool check_repetition(const unsigned long long keycode, const int tolerance) {
	static unsigned int repetition = 0;
	static unsigned long long previous_keycode;
	
	if (keycode == previous_keycode) {
		if (++repetition <= tolerance)
			return false;
	}
	else {
		previous_keycode = keycode;
		repetition = 0;
	}
	return true;
}
